import tempfile

import pytest
from dask_image import imread

from ngff_zarr import (
    from_ngff_zarr,
    to_multiscales,
    to_ngff_image,
    to_ngff_zarr,
)

pytest.importorskip("tensorstore")


def test_from_ngff_zarr(input_images):
    dataset_name = "lung_series"
    data = imread.imread(input_images[dataset_name])
    image = to_ngff_image(
        data=data,
        dims=("z", "y", "x"),
        scale={"z": 2.5, "y": 1.40625, "x": 1.40625},
        translation={"z": 332.5, "y": 360.0, "x": 0.0},
        name="LIDC2",
    )
    multiscales = to_multiscales(image)
    multiscales.scale_factors = None
    multiscales.method = None
    multiscales.chunks = None
    with tempfile.TemporaryDirectory() as tmpdir:
        version = "0.4"
        to_ngff_zarr(tmpdir, multiscales, use_tensorstore=True, version=version)
        multiscales = from_ngff_zarr(tmpdir, version=version, validate=True)
